chrome.runtime.onMessage.addListener(
    function (request, sender, sendResponse) {
        if (request.action === 'startDownload') {
            fetchAndDownloadMods();
        }
    }
);

async function getOperatingSystem(userAgent) {
    if (userAgent.includes('Windows')) return 'Windows';
    else if (userAgent.includes('Linux')) return 'Linux';
    else return 'Other';
}

async function copyToClipboard(text) {
    const textarea = document.createElement('textarea');
    textarea.value = text;
    document.body.appendChild(textarea);
    textarea.select();
    document.execCommand('copy');
    document.body.removeChild(textarea);
}

async function zipAndDownloadFiles(files, zipFileName) {
    const zip = new JSZip();
    await Promise.all(files.map(async (file) => zip.file(file.name, await file.content)));
    const zipContent = await zip.generateAsync({ type: 'blob' });
    const link = document.createElement('a');
    link.href = URL.createObjectURL(zipContent);
    link.download = zipFileName;
    const os = await getOperatingSystem(navigator.userAgent);
    if (os === 'Windows') await copyToClipboard('%appdata%/VintagestoryData/Mods');
    else if (os === 'Linux') await copyToClipboard('~/.config/VintagestoryData/Mods');
    else console.error('Invalid Operating System. The operating system detected is not Windows or Linux.');
    link.click();
    URL.revokeObjectURL(link.href);
}

async function fetchAndDownloadMods() {
    const modElements = Array.from(document.getElementsByClassName('mods')[1].getElementsByClassName('mod'), item => item.getElementsByTagName('a')[0]);
    let finished = 0;
    const totalMods = modElements.length;
    const files = await Promise.all(modElements.map(async (modElement) => {
        const response = await fetch(modElement.href);
        const htmlContent = await response.text();
        const match = htmlContent.match(/<a class="downloadbutton" href="(.+?)">(.+?)<\/a>/);
        const [_, hrefValue, textValue] = match;
        const fileResponse = await fetch(`https://mods.vintagestory.at${hrefValue}`);
        const fileBlob = await fileResponse.blob();
        finished++;
        chrome.runtime.sendMessage({ current: finished, total: totalMods });
        return { name: textValue, content: fileBlob };
    }));

    const validFiles = files.filter(file => file !== null);
    chrome.runtime.sendMessage({ current: 0, total: 0 });
    await zipAndDownloadFiles(validFiles, 'mods.zip');
}